@@ -41,8 +41,6 @@
 #include <pulsecore/thread-mq.h>
 #include <pulsecore/thread.h>
 
-#include "module-null-source-symdef.h"
-
 PA_MODULE_AUTHOR("Lennart Poettering & Marc-Andre Lureau");
 PA_MODULE_DESCRIPTION("Clocked NULL source");
 PA_MODULE_VERSION(PACKAGE_VERSION);
@@ -91,13 +89,6 @@ static int source_process_msg(pa_msgobject *o, int code, void *data, int64_t off
     struct userdata *u = PA_SOURCE(o)->userdata;
 
     switch (code) {
-        case PA_SOURCE_MESSAGE_SET_STATE:
-
-            if (PA_PTR_TO_UINT(data) == PA_SOURCE_RUNNING)
-                u->timestamp = pa_rtclock_now();
-
-            break;
-
         case PA_SOURCE_MESSAGE_GET_LATENCY: {
             pa_usec_t now;
 
@@ -111,6 +102,19 @@ static int source_process_msg(pa_msgobject *o, int code, void *data, int64_t off
     return pa_source_process_msg(o, code, data, offset, chunk);
 }
 
+/* Called from the IO thread. */
+static int source_set_state_in_io_thread_cb(pa_source *s, pa_source_state_t new_state, pa_suspend_cause_t new_suspend_cause) {
+    struct userdata *u;
+
+    pa_assert(s);
+    pa_assert_se(u = s->userdata);
+
+    if (new_state == PA_SOURCE_RUNNING)
+        u->timestamp = pa_rtclock_now();
+
+    return 0;
+}
+
 static void source_update_requested_latency_cb(pa_source *s) {
     struct userdata *u;
 
@@ -231,6 +235,7 @@ int pa__init(pa_module*m) {
     u->latency_time = latency_time;
 
     u->source->parent.process_msg = source_process_msg;
+    u->source->set_state_in_io_thread = source_set_state_in_io_thread_cb;
     u->source->update_requested_latency = source_update_requested_latency_cb;
     u->source->userdata = u;
 
